import { select, brush } from 'd3';

export const main = (container, { state, setState }) => {
  const width = container.clientWidth;
  const height = container.clientHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height)
    .style('background', '#F0FFF4');

  const data = [
    { x: 155, y: 386, r: 20, fill: '#0000FF' },
    { x: 340, y: 238, r: 52, fill: '#FF0AAE' },
    { x: 531, y: 151, r: 20, fill: '#00FF88' },
    { x: 482, y: 307, r: 147, fill: '#7300FF' },
    { x: 781, y: 91, r: 61, fill: '#0FFB33' },
    { x: 668, y: 229, r: 64, fill: '#D400FF' },
  ];
  data.forEach((d, i) => {
    d.id = i;
  });

  const { highlightedSet } = state;

  svg
    .selectAll('circle')
    .data(data)
    .join('circle')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('r', (d) => d.r)
    .attr('fill', (d) => d.fill)
    .attr('stroke', (d) =>
      highlightedSet && highlightedSet.has(d.id)
        ? 'black'
        : 'none',
    )
    .attr('stroke-width', 5)
    .attr('opacity', 708 / 1000);

  const brushBehavior = brush()
    .on('brush', (event) => {
      const [[x0, y0], [x1, y1]] = event.selection;
      const newHighlightedSet = new Set();
      for (const d of data) {
        // If d falls within the brushed rectangle
        // then add it to the highlighted set.
        if (
          d.x >= x0 &&
          d.x <= x1 &&
          d.y >= y0 &&
          d.y <= y1
        ) {
          newHighlightedSet.add(d.id);
        }
      }

      setState((state) => ({
        ...state,
        highlightedSet: newHighlightedSet,
      }));
    })
    .on('end', (event) => {
      if (event.selection === null) {
        setState((state) => ({
          ...state,
          highlightedSet: null,
        }));
      }
    });

  const brushG = svg
    .selectAll('g.brush')
    .data([null])
    .join('g')
    .attr('class', 'brush')
    .call(brushBehavior);
};
